<?php

class CashBill extends PaymentModule {

    const CB_MAIN_URL = "https://pay.cashbill.pl/ws/rest";
    const CB_TEST_URL = "https://pay.cashbill.pl/testws/rest";

    private $urlc_param = array();

    public function __construct() {
        $this->name = 'cashbill';
        $this->tab = 'payments_gateways';
        $this->version = '1.5.4';
        $this->author = 'CashBill';
        $this->currencies = true;
        
        parent::__construct();
        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('CashBill');
        $this->description = $this->l('CashBill online payment system');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall CashBill payment module?');
    }

    public function install() {
        parent::install();

        if (
                Validate::isInt(Configuration::get('PAYMENT_CASHBILL_NEW_STATUS')) XOR ( Validate::isLoadedObject($order_state_new = new OrderState(Configuration::get('PAYMENT_CASHBILL_NEW_STATUS'))))) {
            $order_state_new = new OrderState();
            $order_state_new->name[Language::getIdByIso("pl")] = "Oczekuje potwierdzenia platnosci";
            $order_state_new->name[Language::getIdByIso("en")] = "Awaiting payment confirmation";
            $order_state_new->send_email = false;
            $order_state_new->invoice = false;
            $order_state_new->unremovable = false;
            $order_state_new->color = "lightblue";
            if (!$order_state_new->add())
                return false;
            if (!Configuration::updateValue('PAYMENT_CASHBILL_NEW_STATUS', $order_state_new->id))
                return false;
        }

        if (
                !$this->registerHook('payment') ||
                !$this->registerHook('paymentReturn') ||
                !Configuration::updateValue('CB_ID', '') ||
                !Configuration::updateValue('CB_SECRET_PHRASE', '') ||
                !Configuration::updateValue('CB_TEST_MODE', true)
        ) {
            return false;
        }
    }

    public function uninstall() {
        if (
                !Configuration::deleteByName('CB_ID') ||
                !Configuration::deleteByName('CB_SECRET_PHRASE') ||
                !Configuration::deleteByName('CB_TEST_MODE')
        ) {
            return false;
        }
        return parent::uninstall();
    }

    public function getContent() {
        $configForm = '';
        if (isset($_POST['cb_save'])) {
            Configuration::updateValue('CB_ID', $_POST['cb_id']);
            Configuration::updateValue('CB_SECRET_PHRASE', $_POST['cb_secret_phrase']);
            Configuration::updateValue(
                    'CB_TEST_MODE', isset($_POST['cb_test_mode']) && $_POST['cb_test_mode'] == 'on'
            );
            $configForm = '<div class="conf">
					<img src="../img/admin/ok2.png" />
					Konfiguracja została zapisana
				</div>';
        }
       
        $id = Configuration::get('CB_ID');
        $secretPhrase = Configuration::get('CB_SECRET_PHRASE');
        $testMode = Configuration::get('CB_TEST_MODE') ? 'checked' : '';

        $notifyUrl = "http://{$_SERVER['HTTP_HOST']}" . __PS_BASE_URI__ . "modules/{$this->name}/notify.php";
        $instructionUrl = "http://{$_SERVER['HTTP_HOST']}" . __PS_BASE_URI__ . "modules/{$this->name}/Instrukcja instalacji.pdf";
        $configForm .= <<<EOS
			<form method='post'>
			<fieldset>
				<legend>Konfiguracja punktu płatności</legend>
				<label>Identyfikator</label>			
				<div class="margin-form">
					<input type="text" name="cb_id" value="{$id}" size="70"/>
				</div>
				<label>Klucz</label>
				<div class="margin-form">
					<input type="text" name="cb_secret_phrase" value="{$secretPhrase}" size="70" />
				</div>
				<label>Tryb testowy</label>
				<div class="margin-form">
					<input type="checkbox" {$testMode} name="cb_test_mode" value="on"/>
				</div>
                                <label>Adres serwerowego potwierdzenia transakcji</label>
                                <div class="margin-form">
					<input type="text" value="{$notifyUrl}" readonly size="70" />
				<p class="clear" style="display: block; width: 700px;font-size: 120%;">
W celu umożliwienia dokonywania płatności poprzez system CashBill należy zapoznać się z <a href="{$instructionUrl}" target="_blank"> <img src="../modules/cashbill/pdf-icon.png" alt="Instrukcja Instalacji" height="16" width="16" />Instrukcją instalacji</a>.
			    </p>
					</div>
				<div class="margin-form">
					<input class="button" type='submit' name='cb_save' value='Zapisz'/>
				</div>
    {$this->checkVersion()}
			</fieldset>
			</form>
EOS;
        

        return $configForm;
    }

    public function hookPayment() {
        return $this->display(__FILE__, 'cashbill.tpl');
    }

    public function hookPaymentReturn($status = 'ok') {
        if ($status == 'ok') {
            return $this->display(__FILE__, 'confirmation.tpl');
        } else {
            return $this->display(__FILE__, 'abort.tpl');
        }
    }

    public function execPayment($cart) {
        $address = new Address(intval($cart->id_address_invoice));

        $customer = new Customer(intval($cart->id_customer));
        $currency_info = Currency::getCurrency($cart->id_currency);

        $title = $this->l("Shop ") . Configuration::get('PS_SHOP_NAME');
        $amount = $cart->getOrderTotal();
        $currency = $currency_info["iso_code"];
   		$cart->save();
        
        $this->validateOrder(
               $cart->id, Configuration::get('PAYMENT_CASHBILL_NEW_STATUS'), $amount, $this->displayName, null, array(), null, false, $customer->secure_key);
        
        $orderId = Order::getOrderByCartId($cart->id);
        $returnUrl = "http://{$_SERVER['HTTP_HOST']}" . __PS_BASE_URI__
        . "modules/{$this->name}/confirmation.php?order=".base64_encode(Order::getUniqReferenceOf($orderId));
        $negativeReturnUrl = "http://{$_SERVER['HTTP_HOST']}" . __PS_BASE_URI__
        . "modules/{$this->name}/abort.php?order=".base64_encode(Order::getUniqReferenceOf($orderId));
   
                
        $description = $this->l("order id #") . Order::getUniqReferenceOf($orderId);
        $additionalData = $orderId;
        $languageCode = Language::getIsoById($cart->id_lang);
        $referer = "PrestaShop";
        $firstName = $customer->firstname;
        $surname = $customer->lastname;
        $email = $customer->email;
        $country = $address->country;
        $city = $address->city;
        $postcode = $address->postcode;
        $street = $address->address1;

        $secretPhrase = Configuration::get('CB_SECRET_PHRASE');

        $sign = sha1($title . $amount . $currency . $returnUrl . $description . $negativeReturnUrl
                . $additionalData . $languageCode . $referer . $firstName . $surname
                . $email . $country . $city . $postcode . $street . $secretPhrase);

        $mainUrl = Configuration::get('CB_TEST_MODE') ? self::CB_TEST_URL : self::CB_MAIN_URL;
        $shopId = Configuration::get('CB_ID');

        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_URL => "{$mainUrl}/payment/{$shopId}",
            CURLOPT_POSTFIELDS => http_build_query(array(
                "title" => $title,
                "amount.value" => $amount,
                "amount.currencyCode" => $currency,
                "returnUrl" => $returnUrl,
                "negativeReturnUrl" => $negativeReturnUrl,
                "sign" => $sign,
                "description" => $description,
                "additionalData" => $additionalData,
                "languageCode" => $languageCode,
                "referer" => $referer,
                "personalData.firstName" => $firstName,
                "personalData.surname" => $surname,
                "personalData.email" => $email,
                "personalData.country" => $country,
                "personalData.city" => $city,
                "personalData.postcode" => $postcode,
                "personalData.street" => $street,
            ))
        ));
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/x-www-form-urlencoded; charset=UTF-8"));
        $out = curl_exec($ch);

        if (curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200) {
            curl_close($ch);
            throw new Exception($out);
        }
        curl_close($ch);
       
        $newPayment = json_decode($out);
        header("Location: " . $newPayment->redirectUrl);
    }

    private function getPayment($id) {
        $mainUrl = Configuration::get('CB_TEST_MODE') ? self::CB_TEST_URL : self::CB_MAIN_URL;
        $shopId = Configuration::get('CB_ID');
        $secretPhrase = Configuration::get('CB_SECRET_PHRASE');

        $sign = sha1($id . $secretPhrase);

        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_URL => "{$mainUrl}/payment/{$shopId}/{$id}?sign={$sign}",
        ));

        $out = curl_exec($ch);

        if (curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200) {
            curl_close($ch);
            throw new Exception($out);
        }
        curl_close($ch);

        return json_decode($out);
    }

    public function eventTransactionStatusChanged($id) {

        $transaction = $this->getPayment($id);

        $order = $transaction->additionalData;


        if ($order) {
            if (in_array($transaction->status, array("PositiveFinish", "Abort", "NegativeAuthorization"))) {
                $history = new OrderHistory();
                $history->id_order = $order;

                if (OrderHistory::getLastOrderState($order) !=
                        Configuration::get('PS_OS_PAYMENT')) {
                    if ($transaction->status == "PositiveFinish") {

                        $history->changeIdOrderState(
                                Configuration::get('PS_OS_PAYMENT'), $order);
                    } else {
                        $history->changeIdOrderState(
                                Configuration::get('PS_OS_CANCELED'), $order);
                    }
                    $history->addWithemail(true);
                }
            }
        } else {
            die('UNKNOWN ORDER ID');
        }

        exit("OK");
    }

    public function handleNotification() {
        $cmd = $_GET['cmd'];
        $args = $_GET['args'];
        $sign = $_GET['sign'];

        $secretPhrase = Configuration::get('CB_SECRET_PHRASE');

        if ($sign != md5($cmd . $args . $secretPhrase)) {
            exit("Signature error");
        }

        try {

            switch ($cmd) {
                case 'transactionStatusChanged' :
                    $this->eventTransactionStatusChanged($args);
                    break;

                default :
                    exit("Unknown command");
            }

            exit("OK");
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function checkVersion()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, "http://www.cashbill.pl/ajax/module-version-checker.php?cmd=info&module=Presta%20Shop");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($ch);
        curl_close($ch);
        
        $response = json_decode($response);
        
        if(empty($response))
        {

            return '';
        
        }
        
        if ($response->version == $this->version)         
        { 
            return '<p style="text-align:center;">Posiadasz najnowszą wersję modułu płatności CashBill dla '.$response->name.'</p>';
        }
        
        return '<p style="text-align:center;">Aktualnie posiadasz wersję ' . $this->version . ' jednak jest już dostępna nowsza wersja ' . $response->version . '.</p><p style="text-align:center;"> Możesz ją pobrać korzystając z tego linku <a href="http://www.cashbill.pl/' . $response->link . '">moduł dla ' . $response->name . ' w wersji ' . $response->version . '</a><p>';
    }
    
    
    

}